/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * Programmierbare Relaiskarte mit RS232 untersttzung
 * Die Konfiguration wird im internen EEPROM abgelegt. Insgesamt stehen 512Byte zur Verfgung:
 *  1Byte: Protokoll der genutzten Fernbedienung
 *  2Byte: Gertespezifische Fernbedienungs Adresse
 * 80Byte: Namen der Ports 8*10Byte
 *  1Byte: Anzahl der geschriebenen Datenstze
 *
 *  5Byte: Ein Datensatz bestehend aus:
 *    2Byte: Tastenbefehl
 *    1Byte: On Info
 *    1Byte: Off Info
 *    1Byte: Zustzliche Information die mit der Taste verknpft ist.
 *
 * Insgesamt sollen 44 Tasten Programmiert werden knnen. Dies entspricht dann 200Byte.
 * Es werden dann 284Byte des Speichers reserviert. 228Byte steht noch zur freien Verfgung wenn von einem ATMega8
 * ausgegangen wird.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

#include <stdlib.h>
#include <avr/eeprom.h>
#include <string.h>
#include "irmp.h"
#include "uart.h"

#define UART_BAUD_RATE      19200

#define OUT_PORT	PORTC
#define OUT_DDR		DDRC
#define OUT_1			PC0
#define OUT_2			PC1
#define OUT_3			PC2
#define OUT_4			PC3
#define OUT_5			PC4
#define OUT_6			PC5


#ifndef F_CPU
#error F_CPU unkown
#endif

void timer1_init (void)
{
	#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny85__)              // ATtiny45 / ATtiny85:

	#if F_CPU >= 16000000L
			OCR1C   =  (F_CPU / F_INTERRUPTS / 8) - 1;                            // compare value: 1/15000 of CPU frequency, presc = 8
			TCCR1   = (1 << CTC1) | (1 << CS12);                                  // switch CTC Mode on, set prescaler to 8
	#else
			OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                            // compare value: 1/15000 of CPU frequency, presc = 4
			TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                    // switch CTC Mode on, set prescaler to 4
	#endif

	#else                                                                     // ATmegaXX:
			OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                // compare value: 1/15000 of CPU frequency
			TCCR1B  = (1 << WGM12) | (1 << CS10);                                 // switch CTC Mode on, set prescaler to 1
	#endif

	#ifdef TIMSK1
			TIMSK1  = 1 << OCIE1A;                                                // OCIE1A: Interrupt by timer compare
	#else
			TIMSK   = 1 << OCIE1A;                                                // OCIE1A: Interrupt by timer compare
	#endif
}

#ifdef TIM1_COMPA_vect                                                      // ATtiny84
	#define COMPA_VECT  TIM1_COMPA_vect
	#else
	#define COMPA_VECT  TIMER1_COMPA_vect                                     // ATmega
#endif

ISR(COMPA_VECT)                                                             // Timer1 output compare A interrupt service routine, called every 1/15000 sec
{
  (void) irmp_ISR();                                                        // call irmp ISR
  // call other timer interrupt routines...
}

#define LED_PORT PORTD                                // LED at PD6
#define LED_DDR  DDRD
#define LED_PIN  6


#define CONF_PROTOCOL_ADDR				 0x00//0																			// Adresse im EEPROM um das Protokoll abzulegen (1Byte)
#define CONF_ADDRESS_ADDR					 0x01//1																			// Adresse im EEPROM um die Adresse abzulegen (2Byte)
#define CONF_PORTNAMES_START_ADDR	 0x03//3																			// Start Adresse fr die Port Namen (8x10Byte)
#define CONF_NUM_DATASETS					 0x54//84																			// Adresse im EEPROM; Anzahl der Geschriebenen Datenstze
#define CONF_DATASETS_START_ADDR   0x55//85																			// Startadresse der Tastendatenstze						
#define MAX_DATASETS							40																			// Anzahl der Datenstze die mglich sind
#define MAX_COMMAND_LEN						47																			// Maximale Lnge des zu empfangen Befehls ber RS232
#define MAX_PORTNAME_LEN				  12																			// Lnge des Portnamen
#define MAX_CMD_TOKEN_LEN					20																			// Gre des Befehlstoken
#define MAX_KEY_NAME_LEN					20																			// Lnge des Namens innerhalb eines IR Datensatzes

struct keyInfo_																														// IR Datensatz
{
	//char    name[MAX_KEY_NAME_LEN];																				// Name der Taste
	uint16_t keyCode;																												// Taste auf der Fernbedienung
	uint8_t Options;																												// 1Byte fr zustzliche Informationen
	uint8_t infoOn;																													// On Byte: Hier stehen die On Bits des Ports drin
	uint8_t infoOff;																												// Off Byte: Hier stehen die Off Bits des Ports drin 
};

uint8_t StringLen = 0;																										// Anzahl empfangener Zeichen
char Line[MAX_COMMAND_LEN];																								// die Befehlszeile
uint8_t IR_Protocol = 0xFF;																								// Das verwendete Protokoll;
uint16_t IR_Address = 0xFFFF;																							// Das verwendete Gert;

/*-----------------------------------------------------------------------------------------------------------------------
 * Konvertiert das String Array der Ports zu Port_On und Port_Off
 *-----------------------------------------------------------------------------------------------------------------------
 */
void ConvertToOnOffByte( const char* str, uint8_t* InfoOn, uint8_t* InfoOff )
{
	uint8_t i;

	*InfoOn = 0;
	*InfoOff = 0;

	for( i = 0; i < 8; i++ )
	{
		*InfoOn <<= 1;
		*InfoOff <<= 1;
		
		if( *str == '1' )
		*InfoOn |= 0x01;
		else if( *str == '0' )
		*InfoOff |= 0x01;

		str++;
	}
}

void ConvertToByte( const char* str, uint8_t* InfoByte )
{
	uint8_t i;

	*InfoByte = 0;

	for( i = 0; i < 8; i++ )
	{
		*InfoByte <<= 1;
		
		if( *str == '1' )
		*InfoByte |= 0x01;
		else if( *str == '0' )
		*InfoByte |= 0x00;

		str++;
	}
}

void OnOffByteToChar(char* Str, uint8_t InfoOn, uint8_t InfoOff)
{
	uint8_t i, a = 7;

	for( i = 0; i < 8; i++ )
	{
		if ( (InfoOn & (1 << i)) && !(InfoOff & (1 << i)) )
		{
			Str[a] = '1';
		}
		else if ( !(InfoOn & (1 << i)) && (InfoOff & (1 << i)) )
		{
			Str[a] = '0';
		}
		else
		{
			Str[a] = '-';
		}
		a--;
	}
	Str[8] = '\0';
}

/*-----------------------------------------------------------------------------------------------------------------------
 * Called (back) from IRMP module
 * This example switches a LED (which is connected to Vcc)
 *-----------------------------------------------------------------------------------------------------------------------
 */
void led_callback (uint8_t on)
{
	if (on)
	{
		LED_PORT |= (1 << LED_PIN);
	}
	else
	{
		LED_PORT &= ~(1 << LED_PIN);
	}
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * Aus der empfangenen Befehlszeile wird das Struct erstellt
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void BuildStruct(char* Line, struct keyInfo_* pFound, uint8_t* Prt, uint16_t* Adr)
{
  strtok( Line, ":" );
  *Prt            = strtol( strtok( NULL, ":" ), NULL, 16 );
  *Adr            = strtol( strtok( NULL, ":" ), NULL, 16 );
  pFound->keyCode = strtol( strtok( NULL, ":" ), NULL, 16 );
  ConvertToOnOffByte( strtok( NULL, ";" ), &pFound->infoOn, &pFound->infoOff );
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * Aus der empfangenen Befehlszeile wird das Struct erstellt
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void BuildStruct2(char* Line, struct keyInfo_* pFound, uint8_t* Num)
{
  strtok( Line, ":" );
  *Num            = strtol( strtok( NULL, ":" ), NULL, 16 );
  pFound->keyCode = strtol( strtok( NULL, ":" ), NULL, 16 );
	ConvertToByte( strtok( NULL, ":" ), &pFound->Options );
  ConvertToOnOffByte( strtok( NULL, ";" ), &pFound->infoOn, &pFound->infoOff );
}

/*-----------------------------------------------------------------------------------------------------------------------
 * COMMAND: SET_IR_SYSTEM
 *				  SET_IR_SYSTEM:<Protokoll>:<Adresse>;
 * Schreibt die Fernbedienungsidentifikation ins EEPROM
 *-----------------------------------------------------------------------------------------------------------------------
 */
void CMD_Set_IR_System(char* Line)
{
	uint8_t Protokoll = 0;
	uint16_t Adresse = 0;

	strtok( Line, ":" );
  Protokoll = strtol( strtok( NULL, ":" ), NULL, 16 );
  Adresse = strtol( strtok( NULL, ";" ), NULL, 16 );

	eeprom_update_byte((uint8_t*)CONF_PROTOCOL_ADDR, Protokoll);
	eeprom_update_word((uint16_t*)CONF_ADDRESS_ADDR, Adresse);

	IR_Protocol = Protokoll;
	IR_Address = Adresse;

	uart_puts("SET_IR_SYSTEM:DONE;\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: GET_IR_SYSTEM
						GET_IR_SYSTEM;
	 Liest die Fernbedienungsidentifikationen aus dem EEPROM
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Get_IR_System()
{
	char buf[6];																														// Puffer fr die Konvertieren von Zahlen zu Strings
	uint8_t Protokoll = 0;																									// Variable fr das Protokoll
	uint16_t Adresse = 0;																										// Variable fr die Adresse
				
	Protokoll = eeprom_read_byte((uint8_t*)CONF_PROTOCOL_ADDR);							// Protokoll aus dem EEPROM lesen
	Adresse = eeprom_read_word((uint16_t*)CONF_ADDRESS_ADDR);								// Adresse aus dem EEPROM lesen

	itoa(Protokoll, buf, 16);																								// Protokoll und Adresse in Strings umwandeln
	uart_puts("GET_IR_SYSTEM:");																						// und die Befehlszeile bertragen
	uart_puts(buf);
	uart_putc(':');
	itoa(Adresse, buf, 16);
	uart_puts(buf);
	uart_putc(';');
	uart_puts("\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: GET_NUM_DS
 *					GET_NUM_DS;
 * Liest die Anzahl der geschriebenen Datenstze aus
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Get_Num_DS()
{
	char buf[6];																														// Puffer fr die Konvertieren von Zahlen zu Strings
	uint8_t NumDS = 0;																											// Variable fr die Anzahl der Datenstze
	
	NumDS = eeprom_read_byte((uint8_t*)CONF_NUM_DATASETS);									// Anzahl der Datenstze aus dem EEPROM lesen

	if (NumDS == 0xFF)																												// Wenn Anzahl Datenstze 255 ist
	{																																				// sind 0 Datenstze vorhanden. Damit die
		NumDS = 0x00;																													// Schleife beim auslesen der Datenstze keinen
	}																																				// Mll ausgibt wird er auf 0 gesetzt.

	itoa(NumDS, buf, 16);																										// Protokoll und Adresse in Strings umwandeln
	uart_puts("GET_NUM_DS:");																								// und die Befehlszeile bertragen
	uart_puts(buf);
	uart_putc(';');
	uart_puts("\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: SET_NUM_DS
 *					SET_NUM_DS;
 * Schreibt die Anzahl der geschriebenen Datenstze
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Set_Num_DS(char* Line)
{
	uint8_t NumDS = 0;																											// Variable fr die Anzahl der Datenstze

	strtok( Line, ":" );
	NumDS = strtol( strtok( NULL, ":" ), NULL, 16 );

	eeprom_update_byte((uint8_t*)CONF_NUM_DATASETS, NumDS);

	uart_puts("SET_NUM_DS:DONE;\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: SET_PORTNAME
 *					SET_PORTNAME:<PortNum>:<Name>;
 * Setzt des Namen eines Ports
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Set_PortName(char* Line)
{
	uint8_t PortNum = 0;
	uint16_t PortAddr = 0;
	char PortName[MAX_PORTNAME_LEN];

	strtok( Line, ":" );
	PortNum = strtol( strtok( NULL, ":" ), NULL, 16 );
	strcpy(PortName, strtok( NULL, ";" ) );

	PortAddr = CONF_PORTNAMES_START_ADDR + ((PortNum -1) * MAX_PORTNAME_LEN);
	eeprom_update_block((const void*)PortName, (void*)PortAddr, MAX_PORTNAME_LEN);

	uart_puts("SET_PORTNAME:DONE;\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: GET_PORTNAME
 *					GET_PORTNAME:<PortNum>;
 * lesen des Namen eines Ports
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Get_PortName(char* Line)
{
	uint8_t PortNum = 0;
	uint16_t PortAddr = 0;
	char PortName[MAX_PORTNAME_LEN];
	char buf[6];	

	PortName[0] = '\0';
	strtok( Line, ":" );
	PortNum = strtol( strtok( NULL, ";" ), NULL, 16 );

	PortAddr = CONF_PORTNAMES_START_ADDR + ((PortNum -1) * MAX_PORTNAME_LEN);

	PortName[0] = '\0';
	if (eeprom_read_byte((uint8_t*)PortAddr) != 0xFF)
	{
		eeprom_read_block((void*)PortName, (const void*)PortAddr, MAX_PORTNAME_LEN);
	}

	itoa(PortNum, buf, 10);
	uart_puts("GET_PORTNAME:");
	uart_puts(buf);
	uart_putc(':');
	uart_puts(PortName);
	uart_putc(';');
	uart_puts("\r\n");
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: GET_CONFIG
 *					GET_CONFIG;
 * Auslesen der gesamten Konfiguration
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Get_GetConfig()
{
	uint8_t NumDS = 0, CurrNumDS = 0;
	uint16_t Addr;
	struct keyInfo_ keyDesc;
	char buf[10];

	CurrNumDS = eeprom_read_byte((uint8_t*)CONF_NUM_DATASETS);
	if (CurrNumDS == 0xFF)
	{
		CurrNumDS = 0;
	}

	uart_puts("GET_CONFIG:BEGIN;\r\n");
	uart_puts("COUNT:");
	itoa(CurrNumDS, buf, 10);
	uart_puts(buf);
	uart_puts(";\r\n");

	for (NumDS = 0; NumDS < CurrNumDS; NumDS++)
	{
		Addr = CONF_DATASETS_START_ADDR + ((NumDS) * sizeof(keyDesc));
		eeprom_read_block(&keyDesc, (void*)Addr, sizeof(keyDesc));

		uart_puts("L:");
		itoa(NumDS +1, buf, 10);
		uart_puts(buf);
		uart_putc(':');
		itoa(keyDesc.keyCode, buf, 16);
		uart_puts(buf);
		uart_putc(':');
		itoa(keyDesc.Options, buf, 2);
		uart_puts(buf);
		uart_putc(':');
		OnOffByteToChar(buf, keyDesc.infoOn, keyDesc.infoOff);
		uart_puts(buf);
		uart_puts(";\r\n");
	}

	uart_puts("GET_CONFIG:END;\r\n");
}

void CMD_Set_Key(char* Line)
{
	uint8_t NumDS = 0;
	uint16_t Addr;
	struct keyInfo_ keyDesc;

	BuildStruct2(Line, &keyDesc, &NumDS);
	if (NumDS > 0 && NumDS <= MAX_DATASETS)
	{
		Addr = CONF_DATASETS_START_ADDR + ((NumDS -1) * sizeof(keyDesc));
		eeprom_update_block(&keyDesc, (void*)Addr, sizeof(keyDesc));
		uart_puts("SET_KEY:DONE;\r\n");
	}
	else
	{
		uart_puts("SET_KEY:ERROR;");
	}
}

/*-------------------------------------------------------------------------------------------------------------------------------
 * COMMAND: GET_PORTNAMESS
 *					GET_PORTNAMESS;
 * Auslesen aller Portnamen
 *-------------------------------------------------------------------------------------------------------------------------------
 */
void CMD_Get_PortNames()
{
	uint8_t PortNum;
	uint16_t PortAddr = 0;
	char PortName[MAX_PORTNAME_LEN];
	char buf[6];	

	uart_puts("GET_PORTNAMES:BEGIN;\r\n");
	for (PortNum = 0; PortNum < 8; PortNum++)
	{
		uart_puts("L:");
		itoa(PortNum +1, buf, 10);
		uart_puts(buf);
		uart_putc(':');

		PortAddr = CONF_PORTNAMES_START_ADDR + ((PortNum) * MAX_PORTNAME_LEN);
		PortName[0] = '\0';
		if (eeprom_read_byte((uint8_t*)PortAddr) != 0xFF)
		{
			eeprom_read_block((void*)PortName, (const void*)PortAddr, MAX_PORTNAME_LEN);
		}
		uart_puts(PortName);
		uart_puts(";\r\n");
	}
	uart_puts("GET_PORTNAMES:END;\r\n");
}



void WriteMyPorts(uint8_t infoOn, uint8_t infoOff)
{
	uint8_t i, a = 7;

	for( i = 0; i < 8; i++ )
	{
		if ( (infoOn & (1 << i)) && !(infoOff & (1 << i)) )
		{
			OUT_PORT |= (1 << a);
		}
		else if ( !(infoOn & (1 << i)) && (infoOff & (1 << i)) )
		{
			OUT_PORT &= ~(1 << a);
		}
		else
		{

		}
		a--;
	}
}

void CMD_Set_Ports_Direct(char* Line)
{
	uint8_t infoOn, infoOff;
	char buf[10];

  strtok( Line, ":" );
  ConvertToOnOffByte( strtok( NULL, ";" ), &infoOn, &infoOff );

	//OUT_PORT = ( OUT_PORT | infoOn ) & ( ~ infoOff );
	WriteMyPorts(infoOn, infoOff);
	uart_puts("WRITE::");
	itoa(infoOn, buf, 16);
	uart_puts(buf);
	uart_puts("::");
	itoa(infoOff, buf, 16);
	uart_puts(buf);
	uart_puts("\r\n");
}

/*-----------------------------------------------------------------------------------------------------------------------
 * RS232 Empfangsroutine
 * Empfngt die RS232 Zeichen; Es wird eine Befehlszeile erzeugt die mit der MAX_COMMAND_LEN oder ";" endet;
 * Der Befehlstoken wird geprft und dementsprechend der befehl in einer SUB ausgefhrt. 
 *-----------------------------------------------------------------------------------------------------------------------
 */
void WorkOnCommand()
{
	char temp;																															// Zeichen vom UART
	char Command[MAX_CMD_TOKEN_LEN];																				// Befehlstoken
	uint8_t Index;																													// Index um den Befehlstoken zu identifizieren

	if (uart_data_received())																								// Prfen ob ein Zeichen empfangen wurde
	{
		temp = uart_getc();																										// Empfangenes Zeichen lesen

		if (temp == ';' || StringLen > MAX_COMMAND_LEN -2)										// Wenn das Zeichen kein "Ende" ist oder die Lnge berschreitet dann
		{
			Line[StringLen] = '\0';																							// String terminieren 
			uart_puts("READ: "); uart_puts(Line); uart_puts("\r\n");						// Ausgabe der Empfangenen Befehlszeile

			Index = strcspn(Line, ":");																					// Index des ersten Trenners suchen
			strncpy(Command, Line, Index);																			// Aus der Zeile das Befehlstoken kopieren
			Command[Index] = '\0';																							// string Terminieren

			if (strcmp(Line, "SYS_SYNC") == 0)																	// Synchronisieren des Datenstroms
			{
			}

			if (strcmp(Command, "SET_IR_SYSTEM") == 0)													// Setzen des IR-Systems in den EEPROM
			{
				CMD_Set_IR_System(Line);
			}

			if (strcmp(Command, "GET_IR_SYSTEM") == 0)													// Liest das IR-System fom EEPROM
			{
				CMD_Get_IR_System();
			}

			if (strcmp(Command, "SET_NUM_DS") == 0)															// Anzahl der Datenstze setzen
			{
				CMD_Set_Num_DS(Line);
			}

			if (strcmp(Command, "GET_NUM_DS") == 0)															// Anzahl der Datenstze lese
			{
				CMD_Get_Num_DS();
			}

			if (strcmp(Command, "SET_PORTNAME") == 0)														// Setzen des Portnamen
			{
				CMD_Set_PortName(Line);
			}

			if (strcmp(Command, "GET_PORTNAME") == 0)														// Anzahl der Datenstze lese
			{
				CMD_Get_PortName(Line);
			}

			if (strcmp(Command, "GET_PORTNAMES") == 0)													// Auslesen aller Portnamen
			{
				CMD_Get_PortNames();
			}

			if (strcmp(Command, "GET_CONFIG") == 0)														// Auslesen der Konfiguration
			{
				CMD_Get_GetConfig();
			}

			if (strcmp(Command, "SET_KEY") == 0)
			{
				CMD_Set_Key(Line);
			}

			if (strcmp(Command, "SET_PORTS_DIRECT") == 0)
			{
				CMD_Set_Ports_Direct(Line);
			}

			StringLen = 0;
			Line[0] = '\0';
			uart_puts("\r\n");
		}

		if (temp != ';')
		{
			Line[StringLen] = (unsigned char)temp;
			StringLen++;
		}
	}
}

/*-----------------------------------------------------------------------------------------------------------------------
 * Schreibt den Empfangenen IR Befehl an den PC
 * R:<Protokoll>:<Adresse>:<Befehl>;
 *-----------------------------------------------------------------------------------------------------------------------
 */
void WriteIRDataToUART(IRMP_DATA irmp_data)
{
  char buf[5];
	char Buffer[5];

	uart_puts("R:");
	itoa (irmp_data.protocol, buf, 10);
	if (strlen(buf) == 1)
	{
		Buffer[0] = '0';
		Buffer[1] = buf[0];
		Buffer[2] = '\0';
	}
	uart_puts(Buffer);
	uart_putc(':');
	itoa (irmp_data.address, buf, 16);   
	uart_puts (buf);
	uart_putc (':');
	itoa (irmp_data.command, buf, 16);   
	if (strlen(buf) == 1)
	{
		Buffer[0] = '0';
		Buffer[1] = '0';
		Buffer[2] = '0';
		Buffer[3] = buf[0];
		Buffer[4] = '\0';
	}
	if (strlen(buf) == 2)
	{
		Buffer[0] = '0';
		Buffer[1] = '0';
		Buffer[2] = buf[0];
		Buffer[3] = buf[1];
		Buffer[4] = '\0';
	}
	if (strlen(buf) == 3)
	{
		Buffer[0] = '0';
		Buffer[1] = buf[0];
		Buffer[2] = buf[1];
		Buffer[3] = buf[2];
		Buffer[4] = '\0';
	}
	uart_puts (Buffer);
	uart_putc (';');
	uart_putc ('\r');
	uart_putc ('\n');
}

void Init_IR()
{				
	IR_Protocol = eeprom_read_byte((uint8_t*)CONF_PROTOCOL_ADDR);						// Protokoll aus dem EEPROM lesen
	IR_Address = eeprom_read_word((uint16_t*)CONF_ADDRESS_ADDR);						// Adresse aus dem EEPROM lesen
}

// Test
uint8_t searchKey( uint8_t code, struct keyInfo_* pfound )
{
	struct keyInfo_ keyDesc;
	uint8_t NumDS = 0, CurrNumDS = 0;
	uint16_t Addr;

	CurrNumDS = eeprom_read_byte((uint8_t*)CONF_NUM_DATASETS);
	if (CurrNumDS == 0xFF)
	{
		CurrNumDS = 0;
	}

	for (NumDS = 0; NumDS < CurrNumDS; NumDS++)
	{
		Addr = CONF_DATASETS_START_ADDR + ((NumDS) * sizeof(keyDesc));
		eeprom_read_block(&keyDesc, (void*)Addr, sizeof(keyDesc));

		if (keyDesc.keyCode == code)
		{
			*pfound = keyDesc;
			return TRUE;
		}
	}
	return FALSE;
}

void Handle_IR_Code(void)
{
  IRMP_DATA irmp_data;
  struct keyInfo_ nextInfo;

   if (irmp_get_data(&irmp_data))
   {
    if (irmp_data.flags & IRMP_FLAG_REPETITION)
    {
	    // Benutzer hlt die Taste lnger runter entweder:
	    // ich ignoriere die (Wiederholungs-)Taste oder:
	    // ich benutze diese Info, um einen Repeat-Effekt zu nutzen
			uart_puts("IR::LOOP\r\n");
    }
    else
    {
	    // Es handelt sich um eine neue Taste
	    if (irmp_data.protocol == IR_Protocol && irmp_data.address == IR_Address)						// Nur Befehle vedrarbeiten wenn das IR-System stimmt
	    {
		    WriteIRDataToUART(irmp_data);
			    if (searchKey(irmp_data.command, &nextInfo) == TRUE)
		    {
			    OUT_PORT = ( OUT_PORT | nextInfo.infoOn ) & ( ~ nextInfo.infoOff );
			    //WriteMyPorts(nextInfo.infoOn, nextInfo.infoOff);
			    uart_puts("IR::Set Ports done\r\n");
		    }
		    else
		    {
			    uart_puts("IR::Key not found\r\n");
		    }
	    }
			else
			{
				uart_puts("IR::LOOP\r\n");
			}
    }
   }
}

int main (void)
{
	OUT_DDR |= ((1 << OUT_1) | (1 << OUT_2) | (1 << OUT_3) | (1 << OUT_4) | (1 << OUT_5) | (1 << OUT_6));
	OUT_PORT |= ((1 << OUT_1) | (1 << OUT_2) | (1 << OUT_3) | (1 << OUT_4) | (1 << OUT_5) | (1 << OUT_6));
	_delay_ms(1500);
	OUT_PORT &= ~((1 << OUT_1) | (1 << OUT_2) | (1 << OUT_3) | (1 << OUT_4) | (1 << OUT_5) | (1 << OUT_6));

	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 

  irmp_init();                                                            // initialize irmp
	
  LED_DDR |= (1 << LED_PIN);																							// LED pin to output
  LED_PORT |= (1 << LED_PIN);																							// switch LED off (active low)
  irmp_set_callback_ptr (led_callback);
	
	uart_puts("IR Demo Program\r\n");

	Init_IR();

  timer1_init();                                                          // initialize timer1
  sei ();                                                                 // enable interrupts

  for (;;)																																// Endlosschleife
  {
		WorkOnCommand();																											// Einlesen und verarbeiten der RS232 Befehle

		Handle_IR_Code();
  }
}